@@ -496,6 +496,8 @@ struct codegen_context {
 
 	uint8_t *flag_cache;
 	short *registers;
+	frame_t *var_aux;
+	frame_t *var_aux_2;
 	frame_t *need_spill;
 	size_t need_spill_l;
 
@@ -512,7 +514,6 @@ struct codegen_context {
 	int8_t upcall_args;
 	uint8_t n_pushes;
 	bool upcall_hacked_abi;
-	frame_t *var_aux;
 
 	ajla_error_t err;
 
@@ -541,12 +542,13 @@ static void init_ctx(struct codegen_context *ctx)
 	ctx->args = NULL;
 	ctx->flag_cache = NULL;
 	ctx->registers = NULL;
+	ctx->var_aux = NULL;
+	ctx->var_aux_2 = NULL;
 	ctx->need_spill = NULL;
 	ctx->codegen = NULL;
 	ctx->upcall_offset = -1;
 	ctx->upcall_args = -1;
 	ctx->upcall_hacked_abi = false;
-	ctx->var_aux = NULL;
 }
 
 static void done_ctx(struct codegen_context *ctx)
@@ -591,12 +593,14 @@ static void done_ctx(struct codegen_context *ctx)
 		mem_free(ctx->flag_cache);
 	if (ctx->registers)
 		mem_free(ctx->registers);
+	if (ctx->var_aux)
+		mem_free(ctx->var_aux);
+	if (ctx->var_aux_2)
+		mem_free(ctx->var_aux_2);
 	if (ctx->need_spill)
 		mem_free(ctx->need_spill);
 	if (ctx->codegen)
 		data_free(ctx->codegen);
-	if (ctx->var_aux)
-		mem_free(ctx->var_aux);
 }
 
 
@@ -2202,7 +2206,6 @@ static bool attr_w gen_epilogues(struct codegen_context *ctx)
 #endif
 	if (ctx->reload_label) {
 		gen_label(ctx->reload_label);
-		g(gen_spill_all(ctx));
 		g(gen_mov(ctx, i_size(OP_SIZE_ADDRESS), R_FRAME, R_RET0));
 		g(gen_escape_arg(ctx, (ip_t)-1, nospill_label));
 	}
@@ -2487,6 +2490,10 @@ next_one:;
 	if (unlikely(!ctx->registers))
 		goto fail;
 
+	ctx->var_aux = mem_alloc_array_mayfail(mem_alloc_mayfail, frame_t *, 0, 0, function_n_variables(ctx->fn), sizeof(frame_t), &ctx->err);
+	if (unlikely(!ctx->var_aux))
+		goto fail;
+
 	if (unlikely(!array_init_mayfail(frame_t, &ctx->need_spill, &ctx->need_spill_l, &ctx->err)))
 		goto fail;
 
